use strict;
use warnings;
use FindBin qw($Bin);

&main;
exit;

sub main {
	if ((@ARGV < 2) || (@ARGV>2)) {
		&usage;
	}
	my $env = $ENV{''};
	my $cancer_type = shift(@ARGV);
	my $file_name = shift(@ARGV);
	my @title = ("Chrom","Coordinate","Ref_Allele","Mut_Allele","Strand","HGNC","mRNA","CONSEQUENCE","AAS","AAS_Location","CanDrA_Score","CanDrA_Category","CanDrA_Significance");
	my $file_exist = `if [ -f $Bin/database/$cancer_type/sorted_CanDrA_$cancer_type.gz.tbi ]; then echo "1"; else echo "0"; fi`;
	die("\r\nCanDrA \"$cancer_type\" score table is not completely indexed or the table does not exist\n\r\n") if ($file_exist==0);
	my $file_handle;
	open($file_handle,$file_name) || die ("\r\nCan't open file \"$file_name\".\n\r\n");
	print join("\t",@title)."\r\n";
	while (<$file_handle>) {
		s/[\r\n]+$//;
		if (length($_)<3) {next;}
		my @output_index = split("\t",$_);
		my @detail = @output_index;
		substitution( \@detail );
		my $system_call = `$Bin/tabix-0.2.6/tabix $Bin/database/$cancer_type/sorted_CanDrA_$cancer_type.gz $detail[0]:$detail[1]-$detail[1]`;
		if (!defined($system_call)) {
			next;
		}
		if (length($system_call)==0) {
			next;
		}
		my @records = split("\n",$system_call);
		foreach (@records) {
			my @tmp_record = split("\t",$_);
			if (($tmp_record[3] eq $detail[3])&&($tmp_record[2] eq $detail[2])) {
				print join("\t",@output_index)."\t".join("\t",@tmp_record[4..$#tmp_record])."\r\n";
				last;
			}
		}
	}
	close($file_handle);	
}

sub substitution {
	my $pointer = shift;
	if ($$pointer[4] eq "-") {
		$$pointer[4] = "+";
		$$pointer[3] = single_char_rev($$pointer[3]);
		$$pointer[2] = single_char_rev($$pointer[2]);
	}
}

sub single_char_rev {
	my $char = shift;
	if ( $char eq "A" ) {
		return "T";
	}
	if ( $char eq "T" ) {
		return "A";
	}
	if ( $char eq "C" ) {
		return "G";
	}
	if ( $char eq "G" ) {
		return "C";
	}
}

sub usage {
        die("\r\nProgram:\tCanDrA (Cancer Driver Annotation)\r\nVersion:\t1.0\r\nUsage:\t\tperl open_candra.pl <cancer_type> input_file > output_file\r\n\nOptions:\tcancer_type\tpreset: BRCA, CRC, GBM, MEL, OVC, SCSC\r\n\t\tinput_file\ttab-delimited text file for annotation\r\n\t\toutput_file\tannotated file by candra\r\n\r\nExample:\tperl open_candra.pl OVC demo_input.txt > demo.annotated\r\n\r\n");
}
